/* eslint-disable prefer-template,no-unused-expressions */
import Toast from 'react-native-root-toast' // 引入类库
import { Loading } from '../components/NetworkLoading'
import store from '../index'

const codeMessage = {
  200: '服务器成功返回请求的数据。',
  201: '新建或修改数据成功。',
  202: '一个请求已经进入后台排队（异步任务）。',
  204: '删除数据成功。',
  400: '发出的请求有错误，服务器没有进行新建或修改数据的操作。',
  401: '用户没有权限（令牌、用户名、密码错误）。',
  403: '用户得到授权，但是访问是被禁止的。',
  404: '发出的请求针对的是不存在的记录，服务器没有进行操作。',
  406: '请求的格式不可得。',
  410: '请求的资源被永久删除，且不会再得到的。',
  422: '当创建一个对象时，发生一个验证错误。',
  500: '服务器发生错误，请检查服务器。',
  502: '网关错误。',
  503: '服务不可用，服务器暂时过载或维护。',
  504: '网关超时。',
}

function checkStatus(response) {
  if (response.status >= 200 && response.status < 300) {
    return response
  }
  if (response.status === 403) {
    return response
  }
  const errortext = codeMessage[response.status] || response.statusText
  if (response.status === 401 || response.status === 400) {
    const { dispatch } = store
    dispatch({ type: 'app/logout' })
  }
  Toast.show(errortext, {
    duration: Toast.durations.LONG,
    position: Toast.positions.CENTER,
  })
  // notification.error({
  //   message: `请求错误 ${response.status}: ${response.url}`,
  //   description: errortext,
  // });
  const error = new Error(errortext)
  error.name = response.status
  error.response = response
  throw error
}

export function toParams(param) {
  let result = ''
  for (const name in param) {
    if (typeof param[name] !== 'function') {
      result += '&' + name + '=' + encodeURI(param[name])
    }
  }
  return result.substring(1)
}

/**
 * Requests a URL, returning a promise.
 *
 * @param  {string} url       The URL we want to request
 * @param  {object} [options] The options we want to pass to "fetch"
 * @return {object}           An object containing either "data" or "err"
 */
export default function request(url, options) {
  const defaultOptions = {
    credentials: 'include',
  }
  const newOptions = { ...defaultOptions, ...options }
  if (newOptions.method === 'POST' || newOptions.method === 'PUT') {
    if (!(newOptions.body instanceof FormData)) {
      // if (!(newOptions.body instanceof Object)) {
      newOptions.headers = {
        Accept: 'application/json',
        // 'Content-Type': 'application/x-www-form-urlencoded',
        'Content-Type':
          newOptions.customType === 'application/x-www-form-urlencoded'
            ? 'application/x-www-form-urlencoded'
            : 'application/json;charset=UTF-8',
        ...newOptions.headers,
      }
      newOptions.body =
        newOptions.customType === 'application/x-www-form-urlencoded'
          ? toParams(newOptions.body)
          : JSON.stringify(newOptions.body)
      // newOptions.body = toParams(newOptions.body);
    } else {
      // newOptions.body is FormData
      newOptions.headers = {
        Accept: 'application/json',
        ...newOptions.headers,
      }
    }
  }

  const timer = setTimeout(() => {
    Loading.hidden()
    Toast.show('请求超时，请烧后重试！', {
      duration: Toast.durations.SHORT,
      position: Toast.positions.CENTER,
    })
  }, 30000)

  return fetch(url, newOptions)
    .then(checkStatus)
    .then(response => {
      timer && clearTimeout(timer)
      if (newOptions.method === 'DELETE' || response.status === 204) {
        return response.text()
      }
      if (newOptions.customType === 'application/x-www-form-urlencoded') {
        return response
      }
      return response.json()
    })
    .catch(e => {
      console.log('e:', e)
      timer && clearTimeout(timer)
      Loading.hidden()
      // const { dispatch } = store;
      // const status = e.name;
      // if (status === 401) {
      //   dispatch({
      //     type: 'login/logout',
      //   });
      //   return;
      // }
      // if (status === 403) {
      //   dispatch(routerRedux.push('/exception/403'));
      //   return;
      // }
      // if (status <= 504 && status >= 500) {
      //   dispatch(routerRedux.push('/exception/500'));
      //   return;
      // }
      // if (status >= 404 && status < 422) {
      //   dispatch(routerRedux.push('/exception/404'));
      // }
    })
}
